#include <gtk/gtk.h>
#include <cairo.h>
#include <math.h>

/**** sample: interval timer :begin ****/
#include <time.h>
#include <sys/time.h>
/**** sample: interval timer :end ****/

#include "common.h"

#include "mma8452.h"

#define I2C_ADDR	0x1c /* Device Address(when SA0=0) 0011100=0x1c */
#define I2C_DEVICE	"/dev/i2c-1"

static gint timeout_tag;
static struct mma8452 *acc;

/**** sample: Acceleration data: begin ****/
#define ACC_DATA_SIZE 120

static struct {
	float acc_x;
	float acc_y;
	float acc_z;
	long time_s;
	long time_us;
} acc_data[ACC_DATA_SIZE];

static int acc_cnt = 0;
/**** sample: Acceleration data: end ****/

static void task_set_activate_callback(struct task *task,
				       activate_callback callback,
				       void *data)
{
	task->activate = callback;
	task->activate_data = data;
}

static void task_set_deactivate_callback(struct task *task,
					 deactivate_callback callback,
					 void *data)
{
	task->deactivate = callback;
	task->deactivate_data = data;
}

static int c2toi_12bit(int c2){
	int ret;

	if(c2 > 0x800) ret = -(0x1000 - c2);
	else ret = c2;

	return ret;
}

static gint acc_event(gpointer data)
{
	GtkWidget *draw_area = data;
	int i;

	/**** MMA8452: variable : begin ****/
	uint8_t digital_code[64];
	int xc, yc, zc;
	int ret;
	/**** MMA8452: variable: end ****/

	/**** Log: variable : begin ****/
	FILE *fp;
    	struct timeval time;
	/**** Log: variable : end ****/

	/**** MMA8452: variable initialize: begin ****/
	for(i = 0; i < 64; i++) digital_code[i] = 0x00;
	if(acc_cnt == 0) {
		for(i = 0; i < ACC_DATA_SIZE; i++) {
			acc_data[i].acc_x = 0;
			acc_data[i].acc_y = 0;
			acc_data[i].acc_z = 0;
			acc_data[i].time_s = 0;
			acc_data[i].time_us = 0;
		}
	}
	/**** MMA8452: variable initialize: end ****/

	/**** Log: timestamp : begin ****/
    	gettimeofday(&time, NULL);
	acc_data[acc_cnt].time_s = time.tv_sec;
	acc_data[acc_cnt].time_us = time.tv_usec;
	/**** Log: timestamp : end ****/

	/**** MMA8452: Read Acceleration: begin ****/
	ret = mma8452_read(acc, 0x00, 7, digital_code);
	if(ret !=0) error_msg("mma8452_read", NULL);
	/**** MMA8452: Read Acceleration: end ****/

	/**** MMA8452: Calcurate Acceleration: begin ****/
	xc = ((digital_code[1] << 4) & 0xFF0) + ((digital_code[2] >> 4) & 0x00F);
	yc = ((digital_code[3] << 4) & 0xFF0) + ((digital_code[4] >> 4) & 0x00F);
	zc = ((digital_code[5] << 4) & 0xFF0) + ((digital_code[6] >> 4) & 0x00F);

	acc_data[acc_cnt].acc_x = (float)c2toi_12bit(xc);
	acc_data[acc_cnt].acc_y = (float)c2toi_12bit(yc);
	acc_data[acc_cnt].acc_z = (float)c2toi_12bit(zc);
	acc_cnt++;
	/**** MMA8452: Calcurate Acceleration: end ****/

	if(acc_cnt == 20) {
		gdk_window_invalidate_rect(draw_area->window, NULL, FALSE);

		fp = fopen("/home/www-data/log_acc.txt","w");

		for(i = 0; i < 20; i++) {
			fprintf(fp, "acc=(%5d, %5d, %5d)::time=%10ld[s](%6ld[us])\n",
				(int)acc_data[i].acc_x, 
				(int)acc_data[i].acc_y, 
				(int)acc_data[i].acc_z, 
				acc_data[i].time_s,
				acc_data[i].time_us);
		}

		fclose(fp);

		acc_cnt = 0;
	}

	return TRUE;
}

static void acc_graph_activate(gpointer *data)
{
	uint8_t digital_code[64];
	int ret;

	/**** MMA8452: Device File: open ****/
	acc = mma8452_open(I2C_DEVICE, I2C_ADDR);
	if(acc == NULL) error_msg("mma8452_open", NULL);

	/**** MMA8452: Write Register(CTRL_REG1: 0x2A) for ACTIVE: begin ****/
	digital_code[0] = 0x01; /* CTRL_REG1[0] = 1: ACTIVE */
	ret = mma8452_write(acc, 0x2A, 1, digital_code);
	if(ret !=0) error_msg("mma8452_write", NULL);
	/**** MMA8452: Write Register(CTRL_REG1: 0x2A) for ACTIVE: end ****/

	/**** Timeout: 50ms for Monitor MMA8452: add ****/
	timeout_tag = gtk_timeout_add(50, acc_event, data);
}

static void acc_graph_deactivate(gpointer *data)
{
	/**** Timeout: 50ms for Monitor MMA8452: remove ****/
	gtk_timeout_remove(timeout_tag);

	/**** MMA8452: Device File: close ****/
	mma8452_close(acc);
}

static gboolean acc_graph_expose_event(GtkWidget *widget, GdkEventExpose *event,
				    gpointer data)
{
	cairo_t *cr;
	int i;

	/**** Cairo: variable: begin ****/
	int point_g;
	/**** Cairo: variable: end ****/

	/**** Cairo: Draw: begin ****/
	cr = gdk_cairo_create(widget->window);
	cairo_set_source_rgb(cr, 1.0, 1.0, 1.0);
	cairo_paint(cr);

	/**** Cairo: Draw Axis: begin ****/
	cairo_set_source_rgb(cr, 0.0, 0.0, 0.0);
	cairo_move_to(cr, 50, 20);
	cairo_line_to(cr, 250, 20);
	cairo_stroke(cr);

	cairo_move_to(cr, 50, 120);
	cairo_line_to(cr, 250, 120);	
	cairo_stroke(cr);

	cairo_move_to(cr, 50, 220);
	cairo_line_to(cr, 250, 220);
	cairo_stroke(cr);

	cairo_set_font_size(cr, 12);
	cairo_move_to(cr, 10, 25);
	cairo_show_text(cr, "2048");

	cairo_move_to(cr, 30, 125);
	cairo_show_text(cr, "0");

	cairo_move_to(cr, 10, 225);
	cairo_show_text(cr, "-2048");
	/**** Cairo: Draw Axis: end ****/

	/**** Cairo: Draw Acceleration Graph(X): begin ****/
	cairo_set_source_rgb(cr, 1.0, 0.0, 0.0);
	cairo_move_to(cr, 40, 120 + acc_data[0].acc_x / 2048 * 100);
	cairo_show_text(cr, "x");
	cairo_move_to(cr, 50, 120 + acc_data[0].acc_x / 2048 * 100);
	for(i = 1; i < 20; i++) {
		point_g = (int)((double)acc_data[i].acc_x / 2048 * 100);
		cairo_line_to(cr, 50 + i * 10, 120 + point_g);
		cairo_stroke(cr);
		cairo_move_to(cr, 50 + i * 10, 120 + point_g);
	}
	/**** Cairo: Draw Acceleration Graph(X): end ****/

	/**** Cairo: Draw Acceleration Graph(Y): begin ****/
	cairo_set_source_rgb(cr, 0.0, 1.0, 0.0);
	cairo_move_to(cr, 40, 120 + acc_data[0].acc_y / 2048 * 100);
	cairo_show_text(cr, "y");
	cairo_move_to(cr, 50, 120 + acc_data[0].acc_y / 2048 * 100);
	for(i = 1; i < 20; i++) {
		point_g = (int)((double)acc_data[i].acc_y / 2048 * 100);
		cairo_line_to(cr, 50 + i * 10, 120 + point_g);
		cairo_stroke(cr);
		cairo_move_to(cr, 50 + i * 10, 120 + point_g);
	}
	/**** Cairo: Draw Acceleration Graph(Y): end ****/

	/**** Cairo: Draw Acceleration Graph(Z): begin ****/
	cairo_set_source_rgb(cr, 0.0, 0.0, 1.0);
	cairo_move_to(cr, 40, 120 + acc_data[0].acc_z / 2048 * 100);
	cairo_show_text(cr, "z");
	cairo_move_to(cr, 50, 120 + acc_data[0].acc_z / 2048 * 100);
	for(i = 1; i < 20; i++) {
		point_g = (int)((double)acc_data[i].acc_z / 2048 * 100);
		cairo_line_to(cr, 50 + i * 10, 120 + point_g);
		cairo_stroke(cr);
		cairo_move_to(cr, 50 + i * 10, 120 + point_g);
	}
	/**** Cairo: Draw Acceleration Graph(Z): end ****/

	cairo_destroy(cr);
	/**** Cairo: Draw: end ****/

	return TRUE;
}

int task_acc_graph_setup(struct task *task, GtkWidget *box)
{
	GtkWidget *draw_area;

	draw_area = gtk_drawing_area_new();
	gtk_box_pack_start(GTK_BOX(box), draw_area, TRUE, TRUE, 0);
	g_signal_connect(G_OBJECT(draw_area), "expose-event",
			 G_CALLBACK(acc_graph_expose_event), NULL);

	task_set_activate_callback(task, acc_graph_activate, draw_area);
	task_set_deactivate_callback(task, acc_graph_deactivate, draw_area);

	gtk_widget_show_all(box);

	return 0;
}
