#include <gtk/gtk.h>
#include <cairo.h>
#include <math.h>

#include "common.h"

#include "mma8452.h"

#define I2C_ADDR	0x1c /* Device Address(when SA0=0) 0011100=0x1c */
#define I2C_DEVICE	"/dev/i2c-1"

static gint timeout_tag;
static struct mma8452 *acc;

static void task_set_activate_callback(struct task *task,
				       activate_callback callback,
				       void *data)
{
	task->activate = callback;
	task->activate_data = data;
}

static void task_set_deactivate_callback(struct task *task,
					 deactivate_callback callback,
					 void *data)
{
	task->deactivate = callback;
	task->deactivate_data = data;
}

static int c2toi_12bit(int c2){
	int ret;

	if(c2 > 0x800) ret = -(0x1000 - c2);
	else ret = c2;

	return ret;
}

static gint acc_event(gpointer data)
{
	GtkWidget *draw_area = data;

	gdk_window_invalidate_rect(draw_area->window, NULL, FALSE);

	return TRUE;
}

static void acc_real_activate(gpointer *data)
{
	uint8_t digital_code[64];
	int ret;

	/**** MMA8452: Device File: open ****/
	acc = mma8452_open(I2C_DEVICE, I2C_ADDR);
	if(acc == NULL) error_msg("mma8452_open", NULL);

	/**** MMA8452: Write Register(CTRL_REG1: 0x2A) for ACTIVE: begin ****/
	digital_code[0] = 0x01; /* CTRL_REG1[0] = 1: ACTIVE */
	ret = mma8452_write(acc, 0x2A, 1, digital_code);
	if(ret !=0) error_msg("mma8452_write", NULL);
	/**** MMA8452: Write Register(CTRL_REG1: 0x2A) for ACTIVE: end ****/

	/**** Timeout: 50ms for Monitor MMA8452: add ****/
	timeout_tag = gtk_timeout_add(50, acc_event, data);
}

static void acc_real_deactivate(gpointer *data)
{
	/**** Timeout: 50ms for Monitor MMA8452: remove ****/
	gtk_timeout_remove(timeout_tag);

	/**** MMA8452: Device File: close ****/
	mma8452_close(acc);
}

static gboolean acc_real_expose_event(GtkWidget *widget, GdkEventExpose *event,
				    gpointer data)
{
	cairo_t *cr;
	int i;

	/**** Draw: variable : begin ****/
	int x, y;
	double radx = 20 * M_PI / 180.0;
	double rady = 340 * M_PI / 180.0;
	/**** Draw: variable : end ****/

	/**** MMA8452: variable : begin ****/
	uint8_t digital_code[64];
	int xc, yc, zc, xi, yi, zi;
	char xs[10], ys[10], zs[10];
	int ret;
	/**** MMA8452: variable: end ****/

	/**** MMA8452: variable initialize: begin ****/
	for(i = 0; i < 64; i++) digital_code[i] = 0x00;
	for(i = 0; i < 10; i++) {
		xs[i] = 0x00;
		ys[i] = 0x00;
		zs[i] = 0x00;
	}
	/**** MMA8452: variable initialize: end ****/

	/**** MMA8452: Read Acceleration: begin ****/
	ret = mma8452_read(acc, 0x00, 7, digital_code);
	if(ret !=0) error_msg("mma8452_read", NULL);
	/**** MMA8452: Read Acceleration: end ****/

	/**** MMA8452: Calcurate Acceleration: begin ****/
	xc = ((digital_code[1] << 4) & 0xFF0) + ((digital_code[2] >> 4) & 0x00F);
	yc = ((digital_code[3] << 4) & 0xFF0) + ((digital_code[4] >> 4) & 0x00F);
	zc = ((digital_code[5] << 4) & 0xFF0) + ((digital_code[6] >> 4) & 0x00F);

	xi = c2toi_12bit(xc);
	yi = c2toi_12bit(yc);
	zi = c2toi_12bit(zc);
	/**** MMA8452: Calcurate Acceleration: end ****/

	/**** Draw: Prepare Text: begin ****/
	sprintf(xs, "X: %5d", xi);
	sprintf(ys, "Y: %5d", yi);
	sprintf(zs, "Z: %5d", zi);
	/**** Draw: Prepare Text: end ****/

	/**** Draw: Prepare Point Position: begin ****/
	xi = (int)((double)xi / 2048 * 100);
	yi = (int)((double)yi / 2048 * 100);
	zi = (int)((double)zi / 2048 * 100);

	x = 120 + xi * cos(radx) + yi * cos(rady);
	y = 120 + xi * sin(radx) + yi * sin(rady) - zi;
	/**** Draw: Prepare Point Position: end ****/

	/**** Cairo: Draw: begin ****/
	cr = gdk_cairo_create(widget->window);
	cairo_set_source_rgb(cr, 1.0, 1.0, 1.0);
	cairo_paint(cr);

	/**** Cairo: Draw Axis: begin ****/
	cairo_set_source_rgb(cr, 0.0, 0.0, 0.0);
	cairo_move_to(cr, 120 - (int)(100 * cos(radx)), 120 - (int)(100 * sin(radx)));
	cairo_line_to(cr, 120 + (int)(100 * cos(radx)), 120 + (int)(100 * sin(radx)));
	cairo_stroke(cr);

	cairo_move_to(cr, 120 + (int)(100 * cos(rady)), 120 + (int)(100 * sin(rady)));
	cairo_line_to(cr, 120 - (int)(100 * cos(rady)), 120 - (int)(100 * sin(rady)));
	cairo_stroke(cr);

	cairo_move_to(cr, 120, 120 + 100);
	cairo_line_to(cr, 120, 120 - 100);
	cairo_stroke(cr);

	cairo_set_font_size(cr, 12);
	cairo_move_to(cr, 120 - (int)(100 * cos(radx)), 115 - (int)(100 * sin(radx)));
	cairo_show_text(cr, "x-");
	cairo_move_to(cr, 120 + (int)(100 * cos(radx)), 115 + (int)(100 * sin(radx)));
	cairo_show_text(cr, "x+");

	cairo_move_to(cr, 120 + (int)(100 * cos(rady)), 115 + (int)(100 * sin(rady)));
	cairo_show_text(cr, "y-");
	cairo_move_to(cr, 120 - (int)(100 * cos(rady)), 115 - (int)(100 * sin(rady)));
	cairo_show_text(cr, "y+");

	cairo_move_to(cr, 125, 120 + 100);
	cairo_show_text(cr, "z-");
	cairo_move_to(cr, 125, 120 - 100);
	cairo_show_text(cr, "z+");
	/**** Cairo: Draw Axis: end ****/

	/**** Cairo: Draw Acceleration Point: begin ****/
	cairo_set_source_rgb(cr, 1.0, 0.0, 0.0);
	cairo_arc(cr, x, y, 5, 0, 2 * M_PI);
	cairo_fill(cr);
	/**** Cairo: Draw Acceleration Point: end ****/

	/**** Cairo: Draw Acceleration Text: begin ****/
	cairo_set_font_size(cr, 18);
	cairo_move_to(cr, 250, 20);
	cairo_show_text(cr, xs);
	cairo_move_to(cr, 250, 40);
	cairo_show_text(cr, ys);
	cairo_move_to(cr, 250, 60);
	cairo_show_text(cr, zs);
	/**** Cairo: Draw Acceleration Text: end ****/

	cairo_destroy(cr);
	/**** Cairo: Draw: end ****/

	return TRUE;
}

int task_acc_real_setup(struct task *task, GtkWidget *box)
{
	GtkWidget *draw_area;

	draw_area = gtk_drawing_area_new();
	gtk_box_pack_start(GTK_BOX(box), draw_area, TRUE, TRUE, 0);
	g_signal_connect(G_OBJECT(draw_area), "expose-event",
			 G_CALLBACK(acc_real_expose_event), NULL);

	task_set_activate_callback(task, acc_real_activate, draw_area);
	task_set_deactivate_callback(task, acc_real_deactivate, draw_area);

	gtk_widget_show_all(box);

	return 0;
}
